@@ -1,15 +1,14 @@
 #include "first.h"
 
-#include "server.h"
+#include "base.h"
+#include "burl.h"
 #include "fdevent.h"
+#include "keyvalue.h"
 #include "log.h"
 #include "stream.h"
-#include "plugin.h"
 
 #include "configparser.h"
 #include "configfile.h"
-#include "proc_open.h"
-#include "request.h"
 #include "stat_cache.h"
 
 #include <sys/stat.h>
@@ -65,10 +64,120 @@ static void config_warn_openssl_module (server *srv) {
 }
 #endif
 
+static int config_http_parseopts (server *srv, array *a) {
+    unsigned short int opts = srv->srvconf.http_url_normalize;
+    unsigned short int decode_2f = 1;
+    int rc = 1;
+    if (!array_is_kvstring(a)) {
+        log_error_write(srv, __FILE__, __LINE__, "s",
+                        "unexpected value for server.http-parseopts; "
+                        "expected list of \"key\" => \"[enable|disable]\"");
+        return 0;
+    }
+    for (size_t i = 0; i < a->used; ++i) {
+        const data_string * const ds = (data_string *)a->data[i];
+        unsigned short int opt;
+        int val = 0;
+        if (buffer_is_equal_string(ds->value, CONST_STR_LEN("enable")))
+            val = 1;
+        else if (buffer_is_equal_string(ds->value, CONST_STR_LEN("disable")))
+            val = 0;
+        else {
+            log_error_write(srv, __FILE__, __LINE__, "sbsbs",
+                            "unrecognized value for server.http-parseopts:",
+                            ds->key, "=>", ds->value,
+                            "(expect \"[enable|disable]\")");
+            rc = 0;
+        }
+        if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-normalize")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-normalize-unreserved")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_UNRESERVED;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-normalize-required")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_REQUIRED;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-ctrls-reject")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_CTRLS_REJECT;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-path-backslash-trans")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_PATH_BACKSLASH_TRANS;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-path-2f-decode")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_DECODE;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-path-2f-reject")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_REJECT;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-path-dotseg-remove")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_PATH_DOTSEG_REMOVE;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-path-dotseg-reject")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_PATH_DOTSEG_REJECT;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("url-query-20-plus")))
+            opt = HTTP_PARSEOPT_URL_NORMALIZE_QUERY_20_PLUS;
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("header-strict"))) {
+            srv->srvconf.http_header_strict = val;
+            continue;
+        }
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("host-strict"))) {
+            srv->srvconf.http_host_strict = val;
+            continue;
+        }
+        else if (buffer_is_equal_string(ds->key, CONST_STR_LEN("host-normalize"))) {
+            srv->srvconf.http_host_normalize = val;
+            continue;
+        }
+        else {
+            log_error_write(srv, __FILE__, __LINE__, "sb",
+                            "unrecognized key for server.http-parseopts:",
+                            ds->key);
+            rc = 0;
+            continue;
+        }
+        if (val)
+            opts |= opt;
+        else {
+            opts &= ~opt;
+            if (opt == HTTP_PARSEOPT_URL_NORMALIZE) {
+                opts = 0;
+                break;
+            }
+            if (opt == HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_DECODE) {
+                decode_2f = 0;
+            }
+        }
+    }
+    if (opts != 0) {
+        opts |= HTTP_PARSEOPT_URL_NORMALIZE;
+        if ((opts & (HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_DECODE
+                    |HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_REJECT))
+                 == (HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_DECODE
+                    |HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_REJECT)) {
+            log_error_write(srv, __FILE__, __LINE__, "s",
+                            "conflicting options in server.http-parseopts:"
+                            "url-path-2f-decode, url-path-2f-reject");
+            rc = 0;
+        }
+        if ((opts & (HTTP_PARSEOPT_URL_NORMALIZE_PATH_DOTSEG_REMOVE
+                    |HTTP_PARSEOPT_URL_NORMALIZE_PATH_DOTSEG_REJECT))
+                 == (HTTP_PARSEOPT_URL_NORMALIZE_PATH_DOTSEG_REMOVE
+                    |HTTP_PARSEOPT_URL_NORMALIZE_PATH_DOTSEG_REJECT)) {
+            log_error_write(srv, __FILE__, __LINE__, "s",
+                            "conflicting options in server.http-parseopts:"
+                            "url-path-dotseg-remove, url-path-dotseg-reject");
+            rc = 0;
+        }
+        if (!(opts & (HTTP_PARSEOPT_URL_NORMALIZE_UNRESERVED
+                     |HTTP_PARSEOPT_URL_NORMALIZE_REQUIRED))) {
+            opts |= HTTP_PARSEOPT_URL_NORMALIZE_UNRESERVED;
+            if (decode_2f
+                && !(opts & HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_REJECT))
+                opts |= HTTP_PARSEOPT_URL_NORMALIZE_PATH_2F_DECODE;
+        }
+    }
+    srv->srvconf.http_url_normalize = opts;
+    return rc;
+}
+
 static int config_insert(server *srv) {
 	size_t i;
 	int ret = 0;
 	buffer *stat_cache_string;
+	array *http_parseopts;
 
 	config_values_t cv[] = {
 		{ "server.bind",                       NULL, T_CONFIG_STRING,  T_CONFIG_SCOPE_SERVER     }, /* 0 */
@@ -166,6 +275,7 @@ static int config_insert(server *srv) {
 		{ "server.error-intercept",            NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 79 */
 		{ "server.syslog-facility",            NULL, T_CONFIG_STRING,  T_CONFIG_SCOPE_SERVER     }, /* 80 */
 		{ "server.socket-perms",               NULL, T_CONFIG_STRING,  T_CONFIG_SCOPE_CONNECTION }, /* 81 */
+		{ "server.http-parseopts",             NULL, T_CONFIG_ARRAY,   T_CONFIG_SCOPE_SERVER     }, /* 82 */
 
 		{ NULL,                                NULL, T_CONFIG_UNSET,   T_CONFIG_SCOPE_UNSET      }
 	};
@@ -206,6 +316,8 @@ static int config_insert(server *srv) {
 	cv[74].destination = &(srv->srvconf.http_host_normalize);
 	cv[78].destination = &(srv->srvconf.max_request_field_size);
 	cv[80].destination = srv->srvconf.syslog_facility;
+	http_parseopts = array_init();
+	cv[82].destination = http_parseopts;
 
 	srv->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 
@@ -214,7 +326,7 @@ static int config_insert(server *srv) {
 -analyzer */
 
 	for (i = 0; i < srv->config_context->used; i++) {
-		data_config const* config = (data_config const*)srv->config_context->data[i];
+		data_config * const config = (data_config *)srv->config_context->data[i];
 		specific_config *s;
 
 		s = calloc(1, sizeof(specific_config));
@@ -348,6 +460,52 @@ static int config_insert(server *srv) {
 					"unexpected value for mimetype.assign; expected list of \"ext\" => \"mimetype\"");
 		}
 
+		if (!buffer_string_is_empty(s->server_tag)) {
+			for (char *t = strchr(s->server_tag->ptr,'\n'); NULL != t; t = strchr(t+2,'\n')) {
+				/* not expecting admin to define multi-line server.tag,
+				 * but ensure server_tag has proper header continuation,
+				 * if needed */
+				off_t off = t - s->server_tag->ptr;
+				size_t len;
+				if (t[1] == ' ' || t[1] == '\t') continue;
+				len = buffer_string_length(s->server_tag);
+				buffer_string_prepare_append(s->server_tag, 1);
+				t = s->server_tag->ptr+off;
+				memmove(t+2, t+1, len - off - 1);
+				t[1] = ' ';
+				buffer_commit(s->server_tag, 1);
+			}
+		}
+
+		if (0 == i) {
+                    if (!config_http_parseopts(srv, http_parseopts)) {
+			ret = HANDLER_ERROR;
+			break;
+                    }
+                }
+
+		if (srv->srvconf.http_url_normalize
+		    && COMP_HTTP_QUERY_STRING == config->comp) {
+			switch(config->cond) {
+			case CONFIG_COND_NE:
+			case CONFIG_COND_EQ:
+				/* (can use this routine as long as it does not perform
+				 *  any regex-specific normalization of first arg) */
+				pcre_keyvalue_burl_normalize_key(config->string, srv->tmp_buf);
+				break;
+			case CONFIG_COND_NOMATCH:
+			case CONFIG_COND_MATCH:
+				pcre_keyvalue_burl_normalize_key(config->string, srv->tmp_buf);
+				if (!data_config_pcre_compile(config)) {
+					ret = HANDLER_ERROR;
+				}
+				break;
+			default:
+				break;
+			}
+			if (HANDLER_ERROR == ret) break;
+		}
+
 #if !(defined HAVE_LIBSSL && defined HAVE_OPENSSL_SSL_H)
 		if (s->ssl_enabled) {
 			log_error_write(srv, __FILE__, __LINE__, "s",
@@ -357,6 +515,7 @@ static int config_insert(server *srv) {
 		}
 #endif
 	}
+	array_free(http_parseopts);
 
 	{
 		specific_config *s = srv->config_storage[0];
@@ -365,6 +524,10 @@ static int config_insert(server *srv) {
 		  |(srv->srvconf.http_host_strict    ?(HTTP_PARSEOPT_HOST_STRICT
 		                                      |HTTP_PARSEOPT_HOST_NORMALIZE):0)
 		  |(srv->srvconf.http_host_normalize ?(HTTP_PARSEOPT_HOST_NORMALIZE):0);
+		s->http_parseopts |= srv->srvconf.http_url_normalize;
+
+		if (s->log_request_handling || s->log_request_header)
+			srv->srvconf.log_request_header_on_error = 1;
 	}
 
 	if (0 != stat_cache_choose_engine(srv, stat_cache_string)) {
@@ -1087,9 +1250,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 
 		return 1;
 	} else if (t->offset < t->size) {
-		fprintf(stderr, "%s.%d: %d, %s\n",
-			__FILE__, __LINE__,
-			tid, token->ptr);
+		log_error_write(srv, __FILE__, __LINE__, "Dsb", tid, ",", token);
 	}
 	return 0;
 }
@@ -1245,9 +1406,10 @@ static char* getCWD(void) {
 
 int config_parse_cmd(server *srv, config_t *context, const char *cmd) {
 	tokenizer_t t;
-	int ret;
+	int ret = 0;
+	FILE *fp;
 	buffer *source;
-	buffer *out;
+	buffer *out = srv->tmp_buf;
 	char *oldpwd;
 
 	if (NULL == (oldpwd = getCWD())) {
@@ -1266,19 +1428,38 @@ int config_parse_cmd(server *srv, config_t *context, const char *cmd) {
 	}
 
 	source = buffer_init_string(cmd);
-	out = buffer_init();
 
-	if (0 != proc_open_buffer(cmd, NULL, out, NULL)) {
-		log_error_write(srv, __FILE__, __LINE__, "sbss",
-			"opening", source, "failed:", strerror(errno));
+	fp = popen(cmd, "r");
+	if (NULL == fp) {
+		log_error_write(srv, __FILE__, __LINE__, "SSss",
+				"popen \"", cmd, "\"failed:", strerror(errno));
 		ret = -1;
-	} else {
+	}
+	else {
+		size_t rd;
+		buffer_string_set_length(out, 0);
+		do {
+			rd = fread(buffer_string_prepare_append(out, 1023), 1, 1023, fp);
+			buffer_commit(out, rd);
+		} while (0 != rd && !ferror(fp));
+		if (0 != rd || !feof(fp)) {
+			log_error_write(srv, __FILE__, __LINE__, "SSss",
+					"fread \"", cmd, "\"failed:", strerror(errno));
+			ret = -1;
+		}
+		if (0 != pclose(fp)) {
+			log_error_write(srv, __FILE__, __LINE__, "SSss",
+					"pclose \"", cmd, "\"failed:", strerror(errno));
+			ret = -1;
+		}
+	}
+
+	if (-1 != ret) {
 		tokenizer_init(&t, source, CONST_BUF_LEN(out));
 		ret = config_parse(srv, context, &t);
 	}
 
 	buffer_free(source);
-	buffer_free(out);
 	if (0 != chdir(oldpwd)) {
 		log_error_write(srv, __FILE__, __LINE__, "sss",
 			"cannot change directory to", oldpwd, strerror(errno));
@@ -1370,6 +1551,8 @@ int config_set_defaults(server *srv) {
 	specific_config *s = srv->config_storage[0];
 	struct stat st1, st2;
 
+	force_assert(sizeof(((connection *)0)->conditional_is_valid) >= COMP_LAST_ELEMENT);
+
 	if (0 != fdevent_config(srv)) return -1;
 
 	if (!buffer_string_is_empty(srv->srvconf.changeroot)) {
