@@ -8,6 +8,7 @@
 #line 5 "../../src/configparser.y"
 
 #include "first.h"
+#include "base.h"
 #include "configfile.h"
 #include "buffer.h"
 #include "array.h"
@@ -15,6 +16,7 @@
 
 #include <ctype.h>
 #include <errno.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -148,7 +150,7 @@ static int configparser_remoteip_normalize_compat(buffer *rvalue) {
 }
 
 
-#line 152 "./configparser.c"
+#line 154 "./configparser.c"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -567,44 +569,44 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 24:
     case 25:
     case 26:
-#line 184 "../../src/configparser.y"
+#line 186 "../../src/configparser.y"
 { buffer_free((yypminor->yy0)); }
-#line 572 "./configparser.c"
+#line 574 "./configparser.c"
       break;
     case 36:
-#line 175 "../../src/configparser.y"
-{ if ((yypminor->yy91)) (yypminor->yy91)->free((yypminor->yy91)); }
-#line 577 "./configparser.c"
-      break;
-    case 37:
-#line 176 "../../src/configparser.y"
-{ if ((yypminor->yy91)) (yypminor->yy91)->free((yypminor->yy91)); }
-#line 582 "./configparser.c"
-      break;
-    case 38:
 #line 177 "../../src/configparser.y"
 { if ((yypminor->yy91)) (yypminor->yy91)->free((yypminor->yy91)); }
-#line 587 "./configparser.c"
+#line 579 "./configparser.c"
+      break;
+    case 37:
+#line 178 "../../src/configparser.y"
+{ if ((yypminor->yy91)) (yypminor->yy91)->free((yypminor->yy91)); }
+#line 584 "./configparser.c"
+      break;
+    case 38:
+#line 179 "../../src/configparser.y"
+{ if ((yypminor->yy91)) (yypminor->yy91)->free((yypminor->yy91)); }
+#line 589 "./configparser.c"
       break;
     case 41:
-#line 178 "../../src/configparser.y"
+#line 180 "../../src/configparser.y"
 { array_free((yypminor->yy42)); }
-#line 592 "./configparser.c"
+#line 594 "./configparser.c"
       break;
     case 42:
-#line 179 "../../src/configparser.y"
+#line 181 "../../src/configparser.y"
 { array_free((yypminor->yy42)); }
-#line 597 "./configparser.c"
+#line 599 "./configparser.c"
       break;
     case 43:
-#line 180 "../../src/configparser.y"
+#line 182 "../../src/configparser.y"
 { buffer_free((yypminor->yy29)); }
-#line 602 "./configparser.c"
+#line 604 "./configparser.c"
       break;
     case 44:
-#line 181 "../../src/configparser.y"
+#line 183 "../../src/configparser.y"
 { buffer_free((yypminor->yy29)); }
-#line 607 "./configparser.c"
+#line 609 "./configparser.c"
       break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
@@ -881,9 +883,9 @@ static void yy_reduce(
         /* No destructor defined for global */
         break;
       case 5:
-#line 157 "../../src/configparser.y"
+#line 159 "../../src/configparser.y"
 { yymsp[-1].minor.yy18 = NULL; }
-#line 886 "./configparser.c"
+#line 888 "./configparser.c"
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 6:
@@ -896,7 +898,7 @@ static void yy_reduce(
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 9:
-#line 186 "../../src/configparser.y"
+#line 188 "../../src/configparser.y"
 {
   if (ctx->ok) {
     buffer_copy_buffer(yymsp[0].minor.yy91->key, yymsp[-2].minor.yy29);
@@ -920,11 +922,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 923 "./configparser.c"
+#line 925 "./configparser.c"
   yy_destructor(2,&yymsp[-1].minor);
         break;
       case 10:
-#line 210 "../../src/configparser.y"
+#line 212 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (strncmp(yymsp[-2].minor.yy29->ptr, "env.", sizeof("env.") - 1) == 0) {
@@ -943,11 +945,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 946 "./configparser.c"
+#line 948 "./configparser.c"
   yy_destructor(3,&yymsp[-1].minor);
         break;
       case 11:
-#line 229 "../../src/configparser.y"
+#line 231 "../../src/configparser.y"
 {
   if (ctx->ok) {
     array *vars = ctx->current->value;
@@ -978,11 +980,11 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 981 "./configparser.c"
+#line 983 "./configparser.c"
   yy_destructor(4,&yymsp[-1].minor);
         break;
       case 12:
-#line 260 "../../src/configparser.y"
+#line 262 "../../src/configparser.y"
 {
   if (strchr(yymsp[0].minor.yy0->ptr, '.') == NULL) {
     yygotominor.yy29 = buffer_init_string("var.");
@@ -994,10 +996,10 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 997 "./configparser.c"
+#line 999 "./configparser.c"
         break;
       case 13:
-#line 272 "../../src/configparser.y"
+#line 274 "../../src/configparser.y"
 {
   yygotominor.yy91 = NULL;
   if (ctx->ok) {
@@ -1012,19 +1014,19 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1015 "./configparser.c"
+#line 1017 "./configparser.c"
   yy_destructor(6,&yymsp[-1].minor);
         break;
       case 14:
-#line 287 "../../src/configparser.y"
+#line 289 "../../src/configparser.y"
 {
   yygotominor.yy91 = yymsp[0].minor.yy91;
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1024 "./configparser.c"
+#line 1026 "./configparser.c"
         break;
       case 15:
-#line 292 "../../src/configparser.y"
+#line 294 "../../src/configparser.y"
 {
   yygotominor.yy91 = NULL;
   if (ctx->ok) {
@@ -1049,20 +1051,20 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy29);
   yymsp[0].minor.yy29 = NULL;
 }
-#line 1052 "./configparser.c"
+#line 1054 "./configparser.c"
         break;
       case 16:
-#line 317 "../../src/configparser.y"
+#line 319 "../../src/configparser.y"
 {
   yygotominor.yy91 = (data_unset *)data_string_init();
   buffer_move(((data_string *)(yygotominor.yy91))->value, yymsp[0].minor.yy0);
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1062 "./configparser.c"
+#line 1064 "./configparser.c"
         break;
       case 17:
-#line 324 "../../src/configparser.y"
+#line 326 "../../src/configparser.y"
 {
   char *endptr;
   yygotominor.yy91 = (data_unset *)data_integer_init();
@@ -1077,39 +1079,39 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 1080 "./configparser.c"
+#line 1082 "./configparser.c"
         break;
       case 18:
-#line 338 "../../src/configparser.y"
+#line 340 "../../src/configparser.y"
 {
   yygotominor.yy91 = (data_unset *)data_array_init();
   array_free(((data_array *)(yygotominor.yy91))->value);
   ((data_array *)(yygotominor.yy91))->value = yymsp[0].minor.yy42;
   yymsp[0].minor.yy42 = NULL;
 }
-#line 1090 "./configparser.c"
+#line 1092 "./configparser.c"
         break;
       case 19:
-#line 344 "../../src/configparser.y"
+#line 346 "../../src/configparser.y"
 {
   yygotominor.yy42 = array_init();
 }
-#line 1097 "./configparser.c"
+#line 1099 "./configparser.c"
   yy_destructor(9,&yymsp[-1].minor);
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 20:
-#line 347 "../../src/configparser.y"
+#line 349 "../../src/configparser.y"
 {
   yygotominor.yy42 = yymsp[-1].minor.yy42;
   yymsp[-1].minor.yy42 = NULL;
 }
-#line 1107 "./configparser.c"
+#line 1109 "./configparser.c"
   yy_destructor(9,&yymsp[-2].minor);
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 21:
-#line 352 "../../src/configparser.y"
+#line 354 "../../src/configparser.y"
 {
   yygotominor.yy42 = NULL;
   if (ctx->ok) {
@@ -1131,20 +1133,20 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1134 "./configparser.c"
+#line 1136 "./configparser.c"
   yy_destructor(11,&yymsp[-1].minor);
         break;
       case 22:
-#line 374 "../../src/configparser.y"
+#line 376 "../../src/configparser.y"
 {
   yygotominor.yy42 = yymsp[-1].minor.yy42;
   yymsp[-1].minor.yy42 = NULL;
 }
-#line 1143 "./configparser.c"
+#line 1145 "./configparser.c"
   yy_destructor(11,&yymsp[0].minor);
         break;
       case 23:
-#line 379 "../../src/configparser.y"
+#line 381 "../../src/configparser.y"
 {
   yygotominor.yy42 = NULL;
   if (ctx->ok) {
@@ -1155,18 +1157,18 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1158 "./configparser.c"
+#line 1160 "./configparser.c"
         break;
       case 24:
-#line 390 "../../src/configparser.y"
+#line 392 "../../src/configparser.y"
 {
   yygotominor.yy91 = yymsp[0].minor.yy91;
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1166 "./configparser.c"
+#line 1168 "./configparser.c"
         break;
       case 25:
-#line 394 "../../src/configparser.y"
+#line 396 "../../src/configparser.y"
 {
   yygotominor.yy91 = NULL;
   if (ctx->ok) {
@@ -1180,7 +1182,7 @@ static void yy_reduce(
   buffer_free(yymsp[-2].minor.yy29);
   yymsp[-2].minor.yy29 = NULL;
 }
-#line 1183 "./configparser.c"
+#line 1185 "./configparser.c"
   yy_destructor(12,&yymsp[-1].minor);
         break;
       case 26:
@@ -1189,31 +1191,31 @@ static void yy_reduce(
       case 27:
         break;
       case 28:
-#line 411 "../../src/configparser.y"
+#line 413 "../../src/configparser.y"
 {
   data_config *dc;
   dc = (data_config *)array_get_element_klen(ctx->srv->config_context, CONST_STR_LEN("global"));
   force_assert(dc);
   configparser_push(ctx, dc, 0);
 }
-#line 1199 "./configparser.c"
+#line 1201 "./configparser.c"
   yy_destructor(13,&yymsp[0].minor);
         break;
       case 29:
-#line 418 "../../src/configparser.y"
+#line 420 "../../src/configparser.y"
 {
   force_assert(ctx->current);
   configparser_pop(ctx);
   force_assert(ctx->current);
 }
-#line 1209 "./configparser.c"
+#line 1211 "./configparser.c"
         /* No destructor defined for globalstart */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 30:
-#line 424 "../../src/configparser.y"
+#line 426 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1232,12 +1234,12 @@ static void yy_reduce(
   yymsp[-3].minor.yy18 = NULL;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1235 "./configparser.c"
+#line 1237 "./configparser.c"
         /* No destructor defined for eols */
   yy_destructor(16,&yymsp[-1].minor);
         break;
       case 31:
-#line 443 "../../src/configparser.y"
+#line 445 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1298,20 +1300,20 @@ static void yy_reduce(
   yymsp[-3].minor.yy18 = NULL;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1301 "./configparser.c"
+#line 1303 "./configparser.c"
         /* No destructor defined for eols */
   yy_destructor(16,&yymsp[-1].minor);
         break;
       case 32:
-#line 504 "../../src/configparser.y"
+#line 506 "../../src/configparser.y"
 {
   yygotominor.yy18 = yymsp[0].minor.yy18;
   yymsp[0].minor.yy18 = NULL;
 }
-#line 1311 "./configparser.c"
+#line 1313 "./configparser.c"
         break;
       case 33:
-#line 509 "../../src/configparser.y"
+#line 511 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1325,14 +1327,14 @@ static void yy_reduce(
     yygotominor.yy18 = cur;
   }
 }
-#line 1328 "./configparser.c"
+#line 1330 "./configparser.c"
         /* No destructor defined for context */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 34:
-#line 523 "../../src/configparser.y"
+#line 525 "../../src/configparser.y"
 {
   yygotominor.yy18 = NULL;
   if (ctx->ok) {
@@ -1346,14 +1348,14 @@ static void yy_reduce(
     yygotominor.yy18 = cur;
   }
 }
-#line 1349 "./configparser.c"
+#line 1351 "./configparser.c"
         /* No destructor defined for context_else */
   yy_destructor(14,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(15,&yymsp[0].minor);
         break;
       case 35:
-#line 537 "../../src/configparser.y"
+#line 539 "../../src/configparser.y"
 {
   data_config *dc;
   buffer *b = NULL, *rvalue, *op = NULL;
@@ -1509,49 +1511,17 @@ static void yy_reduce(
         }
       }
 
+      dc->string = buffer_init_buffer(rvalue);
+
       if (ctx->ok) switch(yymsp[-1].minor.yy53) {
       case CONFIG_COND_NE:
       case CONFIG_COND_EQ:
-        dc->string = buffer_init_buffer(rvalue);
         break;
       case CONFIG_COND_NOMATCH:
       case CONFIG_COND_MATCH: {
-#ifdef HAVE_PCRE_H
-        const char *errptr;
-        int erroff, captures;
-
-        if (NULL == (dc->regex =
-            pcre_compile(rvalue->ptr, 0, &errptr, &erroff, NULL))) {
-          dc->string = buffer_init_string(errptr);
-          dc->cond = CONFIG_COND_UNSET;
-
-          fprintf(stderr, "parsing regex failed: %s -> %s at offset %d\n",
-              rvalue->ptr, errptr, erroff);
-
+        if (!data_config_pcre_compile(dc)) {
           ctx->ok = 0;
-        } else if (NULL == (dc->regex_study =
-            pcre_study(dc->regex, 0, &errptr)) &&
-                   errptr != NULL) {
-          fprintf(stderr, "studying regex failed: %s -> %s\n",
-              rvalue->ptr, errptr);
-          ctx->ok = 0;
-        } else if (0 != (pcre_fullinfo(dc->regex, dc->regex_study, PCRE_INFO_CAPTURECOUNT, &captures))) {
-          fprintf(stderr, "getting capture count for regex failed: %s\n",
-              rvalue->ptr);
-          ctx->ok = 0;
-        } else if (captures > 9) {
-          fprintf(stderr, "Too many captures in regex, use (?:...) instead of (...): %s\n",
-              rvalue->ptr);
-          ctx->ok = 0;
-        } else {
-          dc->string = buffer_init_buffer(rvalue);
         }
-#else
-        fprintf(stderr, "can't handle '$%s[%s] =~ ...' as you compiled without pcre support. \n"
-                        "(perhaps just a missing pcre-devel package ?) \n",
-                        yymsp[-5].minor.yy0->ptr, yymsp[-3].minor.yy29->ptr);
-        ctx->ok = 0;
- #endif
         break;
       }
 
@@ -1579,13 +1549,13 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1582 "./configparser.c"
+#line 1552 "./configparser.c"
   yy_destructor(17,&yymsp[-6].minor);
   yy_destructor(19,&yymsp[-4].minor);
   yy_destructor(20,&yymsp[-2].minor);
         break;
       case 36:
-#line 763 "../../src/configparser.y"
+#line 733 "../../src/configparser.y"
 {
   if (ctx->ok) {
     data_config *dc = data_config_init();
@@ -1596,42 +1566,42 @@ static void yy_reduce(
     configparser_push(ctx, dc, 1);
   }
 }
-#line 1599 "./configparser.c"
+#line 1569 "./configparser.c"
         break;
       case 37:
-#line 774 "../../src/configparser.y"
+#line 744 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_EQ;
 }
-#line 1606 "./configparser.c"
+#line 1576 "./configparser.c"
   yy_destructor(21,&yymsp[0].minor);
         break;
       case 38:
-#line 777 "../../src/configparser.y"
+#line 747 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_MATCH;
 }
-#line 1614 "./configparser.c"
+#line 1584 "./configparser.c"
   yy_destructor(22,&yymsp[0].minor);
         break;
       case 39:
-#line 780 "../../src/configparser.y"
+#line 750 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_NE;
 }
-#line 1622 "./configparser.c"
+#line 1592 "./configparser.c"
   yy_destructor(23,&yymsp[0].minor);
         break;
       case 40:
-#line 783 "../../src/configparser.y"
+#line 753 "../../src/configparser.y"
 {
   yygotominor.yy53 = CONFIG_COND_NOMATCH;
 }
-#line 1630 "./configparser.c"
+#line 1600 "./configparser.c"
   yy_destructor(24,&yymsp[0].minor);
         break;
       case 41:
-#line 787 "../../src/configparser.y"
+#line 757 "../../src/configparser.y"
 {
   yygotominor.yy29 = NULL;
   if (ctx->ok) {
@@ -1649,10 +1619,10 @@ static void yy_reduce(
   if (yymsp[0].minor.yy91) yymsp[0].minor.yy91->free(yymsp[0].minor.yy91);
   yymsp[0].minor.yy91 = NULL;
 }
-#line 1652 "./configparser.c"
+#line 1622 "./configparser.c"
         break;
       case 42:
-#line 805 "../../src/configparser.y"
+#line 775 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_file(ctx->srv, ctx, yymsp[0].minor.yy29->ptr)) {
@@ -1662,11 +1632,11 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy29);
   yymsp[0].minor.yy29 = NULL;
 }
-#line 1665 "./configparser.c"
+#line 1635 "./configparser.c"
   yy_destructor(25,&yymsp[-1].minor);
         break;
       case 43:
-#line 815 "../../src/configparser.y"
+#line 785 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_cmd(ctx->srv, ctx, yymsp[0].minor.yy29->ptr)) {
@@ -1676,7 +1646,7 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy29);
   yymsp[0].minor.yy29 = NULL;
 }
-#line 1679 "./configparser.c"
+#line 1649 "./configparser.c"
   yy_destructor(26,&yymsp[-1].minor);
         break;
   };
@@ -1706,11 +1676,11 @@ static void yy_parse_failed(
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 148 "../../src/configparser.y"
+#line 150 "../../src/configparser.y"
 
   ctx->ok = 0;
 
-#line 1713 "./configparser.c"
+#line 1683 "./configparser.c"
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
