@@ -3,9 +3,11 @@
 #include "sys-socket.h"
 #include "base.h"
 #include "connections.h"
+#include "fdevent.h"
 #include "log.h"
 
 #include <errno.h>
+#include <string.h>
 
 const char *connection_get_state(connection_state_t state) {
 	switch (state) {
