@@ -1,7 +1,7 @@
 #include "first.h"
 
+#include "base.h"
 #include "buffer.h"
-#include "server.h"
 #include "log.h"
 #include "connections.h"
 #include "fdevent.h"
@@ -685,7 +685,6 @@ int connection_reset(server *srv, connection *con) {
 		con->request.x = NULL;
 
 	CLEAN(http_host);
-	CLEAN(http_range);
 	CLEAN(http_content_type);
 #undef CLEAN
 	con->request.content_length = 0;
@@ -916,10 +915,18 @@ static handler_t connection_handle_fdevent(server *srv, void *context, int reven
 		} else if (revents & FDEVENT_HUP) {
 			connection_set_state(srv, con, CON_STATE_ERROR);
 		} else if (revents & FDEVENT_RDHUP) {
+			int events = fdevent_event_get_interest(srv->ev, con->fd);
+			events &= ~(FDEVENT_IN|FDEVENT_RDHUP);
+			con->conf.stream_request_body &= ~(FDEVENT_STREAM_REQUEST_BUFMIN|FDEVENT_STREAM_REQUEST_POLLIN);
+			con->conf.stream_request_body |= FDEVENT_STREAM_REQUEST_POLLRDHUP;
+			con->is_readable = 1; /*(can read 0 for end-of-stream)*/
+			con->keep_alive = 0;
+			if (con->request.content_length < -1) { /*(transparent proxy mode; no more data to read)*/
+				con->request.content_length = con->request_content_queue->bytes_in;
+			}
 			if (sock_addr_get_family(&con->dst_addr) == AF_UNIX) {
 				/* future: will getpeername() on AF_UNIX properly check if still connected? */
-				fdevent_event_clr(srv->ev, &con->fde_ndx, con->fd, FDEVENT_RDHUP);
-				con->keep_alive = 0;
+				fdevent_event_set(srv->ev, &con->fde_ndx, con->fd, events);
 			} else if (fdevent_is_tcp_half_closed(con->fd)) {
 				/* Success of fdevent_is_tcp_half_closed() after FDEVENT_RDHUP indicates TCP FIN received,
 				 * but does not distinguish between client shutdown(fd, SHUT_WR) and client close(fd).
@@ -928,8 +935,8 @@ static handler_t connection_handle_fdevent(server *srv, void *context, int reven
 				 * future: might getpeername() to check for TCP RST on half-closed sockets
 				 * (without FDEVENT_RDHUP interest) when checking for write timeouts
 				 * once a second in server.c, though getpeername() on Windows might not indicate this */
-				fdevent_event_clr(srv->ev, &con->fde_ndx, con->fd, FDEVENT_RDHUP);
-				con->keep_alive = 0;
+				con->conf.stream_request_body |= FDEVENT_STREAM_REQUEST_TCP_FIN;
+				fdevent_event_set(srv->ev, &con->fde_ndx, con->fd, events);
 			} else {
 				/* Failure of fdevent_is_tcp_half_closed() indicates TCP RST
 				 * (or unable to tell (unsupported OS), though should not
@@ -1116,16 +1123,12 @@ connection *connection_accepted(server *srv, server_socket *srv_socket, sock_add
 		con->conditional_is_valid[COMP_SERVER_SOCKET] = 1;
 		con->conditional_is_valid[COMP_HTTP_REMOTE_IP] = 1;
 
-		if (-1 == fdevent_fcntl_set_nb_cloexec_sock(srv->ev, con->fd)) {
-			log_error_write(srv, __FILE__, __LINE__, "ss", "fcntl failed: ", strerror(errno));
-			connection_close(srv, con);
-			return NULL;
-		}
 		buffer_copy_string_len(con->proto, CONST_STR_LEN("http"));
 		if (HANDLER_GO_ON != plugins_call_handle_connection_accept(srv, con)) {
 			connection_close(srv, con);
 			return NULL;
 		}
+		if (con->http_status < 0) connection_set_state(srv, con, CON_STATE_WRITE);
 		return con;
 }
 
