@@ -3,6 +3,7 @@
 #include "gw_backend.h"
 
 #include <sys/types.h>
+#include <sys/stat.h>
 #include "sys-socket.h"
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
@@ -19,6 +20,7 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "base.h"
 #include "array.h"
 #include "buffer.h"
 #include "crc32.h"
@@ -538,7 +540,7 @@ static int gw_spawn_connection(server *srv, gw_host *host, gw_proc *proc, int de
                     }
                 }
             } else {
-                char ** const e = environ;
+                char ** const e = fdevent_environ();
                 for (i = 0; e[i]; ++i) {
                     char *eq;
 
@@ -1057,8 +1059,6 @@ static void gw_restart_dead_procs(server *srv, gw_host *host, int debug) {
 #include "base.h"
 #include "connections.h"
 #include "joblist.h"
-#include "keyvalue.h"
-#include "plugin.h"
 #include "response.h"
 
 
@@ -1269,6 +1269,7 @@ int gw_set_defaults_backend(server *srv, gw_plugin_data *p, data_unset *du, size
                 { "listen-backlog",    NULL, T_CONFIG_INT,   T_CONFIG_SCOPE_CONNECTION },        /* 19 */
                 { "x-sendfile",        NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },      /* 20 */
                 { "x-sendfile-docroot",NULL, T_CONFIG_ARRAY,  T_CONFIG_SCOPE_CONNECTION },       /* 21 */
+                { "tcp-fin-propagate", NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },      /* 22 */
 
                 { NULL,                NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
             };
@@ -1324,6 +1325,7 @@ int gw_set_defaults_backend(server *srv, gw_plugin_data *p, data_unset *du, size
             fcv[19].destination = &(host->listen_backlog);
             fcv[20].destination = &(host->xsendfile_allow);
             fcv[21].destination = host->xsendfile_docroot;
+            fcv[22].destination = &(host->tcp_fin_propagate);
 
             if (0 != config_insert_values_internal(srv, da_host->value, fcv, T_CONFIG_SCOPE_CONNECTION)) {
                 goto error;
@@ -1399,8 +1401,12 @@ int gw_set_defaults_backend(server *srv, gw_plugin_data *p, data_unset *du, size
                     host->port = 80;
                 }
 
-                host->family = (!buffer_string_is_empty(host->host)
-                                && NULL != strchr(host->host->ptr, ':'))
+                if (buffer_string_is_empty(host->host)) {
+                    buffer_copy_string_len(host->host,
+                                           CONST_STR_LEN("127.0.0.1"));
+                }
+
+                host->family = (NULL != strchr(host->host->ptr, ':'))
                   ? AF_INET6
                   : AF_INET;
             }
@@ -1711,6 +1717,23 @@ handler_t gw_connection_reset(server *srv, connection *con, void *p_d) {
 }
 
 
+static void gw_conditional_tcp_fin(server *srv, gw_handler_ctx *hctx) {
+    connection *con = hctx->remote_conn;
+    /*assert(con->conf.stream_request_body & FDEVENT_STREAM_REQUEST_TCP_FIN);*/
+    if (!chunkqueue_is_empty(hctx->wb)) return;
+    if (!hctx->host->tcp_fin_propagate) return;
+    if (hctx->gw_mode == GW_AUTHORIZER) return;
+    if (con->conf.stream_request_body & FDEVENT_STREAM_REQUEST_BACKEND_SHUT_WR)
+        return;
+
+    /* propagate shutdown SHUT_WR to backend if TCP half-close on con->fd */
+    con->conf.stream_request_body |= FDEVENT_STREAM_REQUEST_BACKEND_SHUT_WR;
+    con->conf.stream_request_body &= ~FDEVENT_STREAM_REQUEST_POLLIN;
+    con->is_readable = 0;
+    shutdown(hctx->fd, SHUT_WR);
+    fdevent_event_clr(srv->ev, &hctx->fde_ndx, hctx->fd, FDEVENT_OUT);
+}
+
 static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
     switch(hctx->state) {
     case GW_STATE_INIT:
@@ -1804,7 +1827,7 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
         /*(disable Nagle algorithm if streaming and content-length unknown)*/
         if (AF_UNIX != hctx->host->family) {
             connection *con = hctx->remote_conn;
-            if (-1 == con->request.content_length) {
+            if (con->request.content_length < 0) {
                 if (-1 == fdevent_set_tcp_nodelay(hctx->fd, 1)) {
                     /*(error, but not critical)*/
                 }
@@ -1876,6 +1899,10 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
             }
         }
 
+        if (hctx->remote_conn->conf.stream_request_body
+            & FDEVENT_STREAM_REQUEST_TCP_FIN)
+            gw_conditional_tcp_fin(srv, hctx);
+
         return HANDLER_WAIT_FOR_EVENT;
     case GW_STATE_READ:
         /* waiting for a response */
@@ -1997,10 +2024,18 @@ handler_t gw_handle_subrequest(server *srv, connection *con, void *p_d) {
         }
     }
 
-    return ((0 == hctx->wb->bytes_in || !chunkqueue_is_empty(hctx->wb))
-        && hctx->state != GW_STATE_CONNECT_DELAYED)
-      ? gw_send_request(srv, hctx)
-      : HANDLER_WAIT_FOR_EVENT;
+    {
+        handler_t rc =((0==hctx->wb->bytes_in || !chunkqueue_is_empty(hctx->wb))
+                       && hctx->state != GW_STATE_CONNECT_DELAYED)
+          ? gw_send_request(srv, hctx)
+          : HANDLER_WAIT_FOR_EVENT;
+        if (HANDLER_WAIT_FOR_EVENT != rc) return rc;
+    }
+
+    if (con->conf.stream_request_body & FDEVENT_STREAM_REQUEST_TCP_FIN)
+        gw_conditional_tcp_fin(srv, hctx);
+
+    return HANDLER_WAIT_FOR_EVENT;
 }
 
 
@@ -2013,12 +2048,15 @@ static handler_t gw_recv_response(server *srv, gw_handler_ctx *hctx) {
       ? buffer_init()
       : hctx->response;
 
-    switch (http_response_read(srv, hctx->remote_conn, &hctx->opts,
-                               b, hctx->fd, &hctx->fde_ndx)) {
+    handler_t rc = http_response_read(srv, hctx->remote_conn, &hctx->opts,
+                                      b, hctx->fd, &hctx->fde_ndx);
+
+    if (b != hctx->response) buffer_free(b);
+
+    switch (rc) {
     default:
-        break;
+        return HANDLER_GO_ON;
     case HANDLER_FINISHED:
-        if (b != hctx->response) buffer_free(b);
         if (hctx->gw_mode == GW_AUTHORIZER
             && (200 == con->http_status || 0 == con->http_status)) {
             /*
@@ -2071,7 +2109,6 @@ static handler_t gw_recv_response(server *srv, gw_handler_ctx *hctx) {
         return HANDLER_FINISHED;
     case HANDLER_COMEBACK: /*(not expected; treat as error)*/
     case HANDLER_ERROR:
-        if (b != hctx->response) buffer_free(b);
         /* (optimization to detect backend process exit while processing a
          *  large number of ready events; (this block could be removed)) */
         if (proc->is_local && 1 == proc->load && proc->pid == hctx->pid
@@ -2123,9 +2160,6 @@ static handler_t gw_recv_response(server *srv, gw_handler_ctx *hctx) {
         gw_connection_close(srv, hctx);
         return HANDLER_FINISHED;
     }
-
-    if (b != hctx->response) buffer_free(b);
-    return HANDLER_GO_ON;
 }
 
 
@@ -2164,9 +2198,13 @@ static handler_t gw_handle_fdevent(server *srv, void *ctx, int revents) {
              * since event loop will spin on fd FDEVENT_HUP event
              * until unregistered. */
             handler_t rc;
+            const unsigned short flags = con->conf.stream_response_body;
+            con->conf.stream_response_body &= ~FDEVENT_STREAM_RESPONSE_BUFMIN;
+            con->conf.stream_response_body |= FDEVENT_STREAM_RESPONSE_POLLRDHUP;
             do {
                 rc = gw_recv_response(srv,hctx); /*(might invalidate hctx)*/
             } while (rc == HANDLER_GO_ON);       /*(unless HANDLER_GO_ON)*/
+            con->conf.stream_response_body = flags;
             return rc; /* HANDLER_FINISHED or HANDLER_ERROR */
         } else {
             gw_proc *proc = hctx->proc;
