@@ -27,21 +27,10 @@ These notices must be retained in any copies of any part of this
 documentation and/or software.
  */
 
-#ifdef HAVE_STDINT_H
-# include <stdint.h>
-#endif
-#ifdef HAVE_INTTYPES_H
-# include <inttypes.h>
-#endif
-
-#define UINT4 uint32_t
-#define UINT2 uint16_t
-#define POINTER unsigned char *
-
 /* MD5 context. */
 typedef struct {
-  UINT4 state[4];                                   /* state (ABCD) */
-  UINT4 count[2];        /* number of bits, modulo 2^64 (lsb first) */
+  uint32_t state[4];                                /* state (ABCD) */
+  uint32_t count[2];     /* number of bits, modulo 2^64 (lsb first) */
   unsigned char buffer[64];                         /* input buffer */
 } li_MD5_CTX;
 
