@@ -36,6 +36,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 typedef struct {
     buffer *auth_gssapi_keytab;
@@ -217,10 +218,14 @@ static int mod_authn_gssapi_create_krb5_ccache(server *srv, connection *con, plu
     char * const ccname    = kccname->ptr + sizeof("FILE:")-1;
     const size_t ccnamelen = buffer_string_length(kccname)-(sizeof("FILE:")-1);
     /*(future: might consider using server.upload-dirs instead of /tmp)*/
+  #ifdef __COVERITY__
+    /* POSIX-2008 requires mkstemp create file with 0600 perms */
+    umask(0600);
+  #endif
     /* coverity[secure_temp : FALSE] */
     int fd = mkstemp(ccname);
     if (fd < 0) {
-        log_error_write(srv, __FILE__, __LINE__, "ss", "mkstemp():", strerror(errno));
+        log_error_write(srv, __FILE__, __LINE__, "sss", "mkstemp():", ccname, strerror(errno));
         buffer_free(kccname);
         return -1;
     }
