@@ -107,8 +107,10 @@
 #include <string.h>
 #include <errno.h>
 #include <time.h>
+#include <unistd.h>     /* read() */
 
 #include "base.h"
+#include "fdevent.h"
 #include "log.h"
 #include "buffer.h"
 #include "etag.h"
@@ -324,6 +326,18 @@ SETDEFAULTS_FUNC(mod_deflate_setdefaults) {
 			s->max_loadavg = strtod(p->tmp_buf->ptr, NULL);
 		}
 
+		if (!array_is_vlist(s->mimetypes)) {
+			log_error_write(srv, __FILE__, __LINE__, "s",
+					"unexpected value for deflate.mimetypes; expected list of \"mimetype\"");
+			return HANDLER_ERROR;
+		}
+
+		if (!array_is_vlist(p->encodings)) {
+			log_error_write(srv, __FILE__, __LINE__, "s",
+					"unexpected value for deflate.allowed-encodings; expected list of \"encoding\"");
+			return HANDLER_ERROR;
+		}
+
 		if (p->encodings->used) {
 			size_t j = 0;
 			for (j = 0; j < p->encodings->used; j++) {
@@ -778,9 +792,8 @@ static int mod_deflate_file_chunk(server *srv, connection *con, handler_ctx *hct
 		/* we have more to send than we can mmap() at once */
 		if (we_want_to_send > to_mmap) we_want_to_send = to_mmap;
 
-		if (MAP_FAILED == (c->file.mmap.start = mmap(0, (size_t)to_mmap, PROT_READ, MAP_SHARED, c->file.fd, c->file.mmap.offset))) {
-			/* close it here, otherwise we'd have to set FD_CLOEXEC */
-
+		if (MAP_FAILED == (c->file.mmap.start = mmap(0, (size_t)to_mmap, PROT_READ, MAP_SHARED, c->file.fd, c->file.mmap.offset))
+		    && (errno != EINVAL || MAP_FAILED == (c->file.mmap.start = mmap(0, (size_t)to_mmap, PROT_READ, MAP_PRIVATE, c->file.fd, c->file.mmap.offset)))) {
 			log_error_write(srv, __FILE__, __LINE__, "ssbd", "mmap failed:",
 					strerror(errno), c->file.name, c->file.fd);
 
@@ -827,6 +840,7 @@ static int mod_deflate_file_chunk(server *srv, connection *con, handler_ctx *hct
 			free(start);
 			return -1;
 		}
+		abs_offset = 0;
 	}
 
 #ifdef USE_MMAP
@@ -1036,7 +1050,7 @@ CONNECTION_FUNC(mod_deflate_handle_response_start) {
 	/*(current implementation requires response be complete)*/
 	if (!con->file_finished) return HANDLER_GO_ON;
 	if (con->request.http_method == HTTP_METHOD_HEAD) return HANDLER_GO_ON;
-	if (con->parsed_response & HTTP_TRANSFER_ENCODING_CHUNKED) return HANDLER_GO_ON;
+	if (con->parsed_response & HTTP_TRANSFER_ENCODING) return HANDLER_GO_ON;
 
 	/* disable compression for some http status types. */
 	switch(con->http_status) {
@@ -1245,7 +1259,6 @@ int mod_deflate_plugin_init(plugin *p) {
 	p->cleanup	= mod_deflate_free;
 	p->set_defaults	= mod_deflate_setdefaults;
 	p->connection_reset	= mod_deflate_cleanup;
-	p->handle_connection_close	= mod_deflate_cleanup;
 	p->handle_response_start	= mod_deflate_handle_response_start;
 
 	p->data        = NULL;
