@@ -1,10 +1,12 @@
 #include "first.h"
 
+#include "base.h"
 #include "plugin.h"
 #include "log.h"
 #include "response.h"
 #include "stat_cache.h"
 
+#include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 
