@@ -2,6 +2,7 @@
 
 #include <sys/types.h>
 #include <errno.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "gw_backend.h"
@@ -14,7 +15,6 @@ typedef gw_handler_ctx   handler_ctx;
 #include "fdevent.h"
 #include "http_chunk.h"
 #include "log.h"
-#include "plugin.h"
 #include "status_counter.h"
 
 #ifdef HAVE_FASTCGI_FASTCGI_H
@@ -368,7 +368,10 @@ static handler_t fcgi_recv_parse(server *srv, connection *con, struct http_respo
 	int fin = 0;
 
 	if (0 == n) {
-		if (!(fdevent_event_get_interest(srv->ev, hctx->fd) & FDEVENT_IN)) return HANDLER_GO_ON;
+		if (-1 == hctx->request_id) return HANDLER_FINISHED; /*(flag request ended)*/
+		if (!(fdevent_event_get_interest(srv->ev, hctx->fd) & FDEVENT_IN)
+		    && !(con->conf.stream_response_body & FDEVENT_STREAM_RESPONSE_POLLRDHUP))
+			return HANDLER_GO_ON;
 		log_error_write(srv, __FILE__, __LINE__, "ssdsb",
 				"unexpected end-of-file (perhaps the fastcgi process died):",
 				"pid:", hctx->proc->pid,
@@ -436,6 +439,7 @@ static handler_t fcgi_recv_parse(server *srv, connection *con, struct http_respo
 
 			break;
 		case FCGI_END_REQUEST:
+			hctx->request_id = -1; /*(flag request ended)*/
 			fin = 1;
 			break;
 		default:
@@ -459,6 +463,7 @@ static int fcgi_patch_connection(server *srv, connection *con, plugin_data *p) {
 	PATCH(exts_auth);
 	PATCH(exts_resp);
 	PATCH(debug);
+	PATCH(balance);
 	PATCH(ext_mapping);
 
 	/* skip the first, the global context */
@@ -479,6 +484,8 @@ static int fcgi_patch_connection(server *srv, connection *con, plugin_data *p) {
 				PATCH(exts_resp);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("fastcgi.debug"))) {
 				PATCH(debug);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("fastcgi.balance"))) {
+				PATCH(balance);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("fastcgi.map-extensions"))) {
 				PATCH(ext_mapping);
 			}
@@ -508,7 +515,12 @@ static handler_t fcgi_check_extension(server *srv, connection *con, void *p_d, i
 		hctx->opts.pdata = hctx;
 		hctx->stdin_append = fcgi_stdin_append;
 		hctx->create_env = fcgi_create_env;
-		hctx->rb = chunkqueue_init();
+		if (!hctx->rb) {
+			hctx->rb = chunkqueue_init();
+		}
+		else {
+			chunkqueue_reset(hctx->rb);
+		}
 	}
 
 	return HANDLER_GO_ON;
