@@ -3,6 +3,7 @@
 #include <sys/types.h>
 #include <errno.h>
 #include <limits.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "gw_backend.h"
@@ -13,7 +14,6 @@ typedef gw_handler_ctx   handler_ctx;
 #include "base.h"
 #include "buffer.h"
 #include "log.h"
-#include "plugin.h"
 #include "status_counter.h"
 
 #include "sys-endian.h"
@@ -217,6 +217,7 @@ static int scgi_patch_connection(server *srv, connection *con, plugin_data *p) {
 	PATCH(exts_resp);
 	PATCH(proto);
 	PATCH(debug);
+	PATCH(balance);
 	PATCH(ext_mapping);
 
 	/* skip the first, the global context */
@@ -237,6 +238,8 @@ static int scgi_patch_connection(server *srv, connection *con, plugin_data *p) {
 				PATCH(exts_resp);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("scgi.protocol"))) {
 				PATCH(proto);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("scgi.balance"))) {
+				PATCH(balance);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("scgi.debug"))) {
 				PATCH(debug);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("scgi.map-extensions"))) {
