@@ -77,6 +77,7 @@
 #include "first.h"
 
 #include <sys/types.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -89,7 +90,6 @@
 #include "fdevent.h"
 #include "joblist.h"
 #include "log.h"
-#include "plugin.h"
 
 #define MOD_WEBSOCKET_LOG_NONE  0
 #define MOD_WEBSOCKET_LOG_ERR   1
@@ -164,6 +164,7 @@ typedef struct {
 
     int hybivers;
     time_t ping_ts;
+    int subproto;
 
     server *srv;  /*(for mod_wstunnel module-specific DEBUG_LOG() macro)*/
     plugin_config conf;
@@ -423,7 +424,7 @@ static int wstunnel_is_allowed_origin(connection *con, handler_ctx *hctx) {
      * Note that origin provided in request header has not been normalized, so
      * change in case or other non-normal forms might not match allowed list */
     const array * const allowed_origins = hctx->conf.origins;
-    buffer *origin;
+    buffer *origin = NULL;
     size_t olen;
     data_string *dsorigin;
 
@@ -535,9 +536,30 @@ static handler_t wstunnel_handler_setup (server *srv, connection *con, plugin_da
     if (!binary) {
         data_string *ds = (data_string *)
           array_get_element(con->request.headers, "Sec-WebSocket-Protocol");
-        binary = (NULL != ds
-                  && buffer_is_equal_caseless_string(ds->value,
-                                                     CONST_STR_LEN("binary")));
+        if (NULL != ds) {
+            for (const char *s = ds->value->ptr; *s; ++s) {
+                while (*s==' '||*s=='\t'||*s=='\r'||*s=='\n') ++s;
+                if (0 == strncasecmp(s, "binary", sizeof("binary")-1)) {
+                    s += sizeof("binary")-1;
+                    while (*s==' '||*s=='\t'||*s=='\r'||*s=='\n') ++s;
+                    if (*s==','||*s=='\0') {
+                        hctx->subproto = 1;
+                        binary = 1;
+                        break;
+                    }
+                }
+                else if (0 == strncasecmp(s, "base64", sizeof("base64")-1)) {
+                    s += sizeof("base64")-1;
+                    while (*s==' '||*s=='\t'||*s=='\r'||*s=='\n') ++s;
+                    if (*s==','||*s=='\0') {
+                        hctx->subproto = -1;
+                        break;
+                    }
+                }
+                s = strchr(s, ',');
+                if (NULL == s) break;
+            }
+        }
     }
 
     if (binary) {
@@ -622,7 +644,7 @@ TRIGGER_FUNC(mod_wstunnel_handle_trigger) {
 
         if (0 != hctx->hybivers
             && hctx->conf.ping_interval > 0
-            && hctx->conf.ping_interval + hctx->ping_ts < cur_ts) {
+            && (time_t)hctx->conf.ping_interval + hctx->ping_ts < cur_ts) {
             hctx->ping_ts = cur_ts;
             mod_wstunnel_frame_send(hctx, MOD_WEBSOCKET_FRAME_TYPE_PING, CONST_STR_LEN("ping"));
             joblist_append(srv, con);
@@ -843,6 +865,9 @@ static int create_response_rfc_6455(handler_ctx *hctx) {
     if (hctx->frame.type == MOD_WEBSOCKET_FRAME_TYPE_BIN)
         array_set_key_value(hdrs, CONST_STR_LEN("Sec-WebSocket-Protocol"),
                                   CONST_STR_LEN("binary"));
+    else if (-1 == hctx->subproto)
+        array_set_key_value(hdrs, CONST_STR_LEN("Sec-WebSocket-Protocol"),
+                                  CONST_STR_LEN("base64"));
 
     return 0;
 }
