@@ -9,15 +9,21 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
 
-#ifdef USE_OPENSSL
+#if defined HAVE_LIBSSL && defined HAVE_OPENSSL_SSL_H
+#define USE_OPENSSL_CRYPTO
+#endif
+#ifdef USE_OPENSSL_CRYPTO
+#include <openssl/opensslv.h> /* OPENSSL_VERSION_NUMBER */
 #include <openssl/rand.h>
 #endif
+#ifdef HAVE_GETENTROPY
+#include <sys/random.h>
+#endif
 #ifdef HAVE_LINUX_RANDOM_H
 #include <sys/syscall.h>
 #include <linux/random.h>
@@ -107,7 +113,8 @@ static int li_rand_device_bytes (unsigned char *buf, int num)
             ssize_t rd = 0;
           #ifdef RNDGETENTCNT
             int entropy;
-            if (0 == ioctl(fd, RNDGETENTCNT, &entropy) && entropy >= num*8)
+            if (0 == ioctl(fd, (unsigned long)(RNDGETENTCNT), &entropy)
+                && entropy >= num*8)
           #endif
                 rd = read(fd, buf, (size_t)num);
             close(fd);
@@ -154,7 +161,7 @@ static void li_rand_init (void)
   #ifdef HAVE_SRANDOM
     srandom(u); /*(initialize just in case random() used elsewhere)*/
   #endif
-  #ifdef USE_OPENSSL
+  #ifdef USE_OPENSSL_CRYPTO
     RAND_poll();
     RAND_seed(xsubi, (int)sizeof(xsubi));
   #endif
@@ -165,11 +172,11 @@ void li_rand_reseed (void)
     if (li_rand_inited) li_rand_init();
 }
 
-int li_rand_pseudo_bytes (void)
+int li_rand_pseudo (void)
 {
     /* randomness *is not* cryptographically strong */
     /* (attempt to use better mechanisms to replace the more portable rand()) */
-  #ifdef USE_OPENSSL  /* (RAND_pseudo_bytes() is deprecated in openssl 1.1.0) */
+  #ifdef USE_OPENSSL_CRYPTO /* (openssl 1.1.0 deprecates RAND_pseudo_bytes()) */
   #if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
     int i;
     if (-1 != RAND_pseudo_bytes((unsigned char *)&i, sizeof(i))) return i;
@@ -191,9 +198,15 @@ int li_rand_pseudo_bytes (void)
   #endif
 }
 
+void li_rand_pseudo_bytes (unsigned char *buf, int num)
+{
+    for (int i = 0; i < num; ++i)
+        buf[i] = li_rand_pseudo() & 0xFF;
+}
+
 int li_rand_bytes (unsigned char *buf, int num)
 {
-  #ifdef USE_OPENSSL
+  #ifdef USE_OPENSSL_CRYPTO
     int rc = RAND_bytes(buf, num);
     if (-1 != rc) {
         return rc;
@@ -204,8 +217,7 @@ int li_rand_bytes (unsigned char *buf, int num)
     }
     else {
         /* NOTE: not cryptographically random !!! */
-        for (int i = 0; i < num; ++i)
-            buf[i] = li_rand_pseudo_bytes() & 0xFF;
+        li_rand_pseudo_bytes(buf, num);
         /*(openssl RAND_pseudo_bytes rc for non-cryptographically random data)*/
         return 0;
     }
@@ -213,7 +225,7 @@ int li_rand_bytes (unsigned char *buf, int num)
 
 void li_rand_cleanup (void)
 {
-  #ifdef USE_OPENSSL
+  #ifdef USE_OPENSSL_CRYPTO
     RAND_cleanup();
   #endif
     safe_memclear(xsubi, sizeof(xsubi));
