@@ -1,7 +1,9 @@
 #include "first.h"
 
 #include "request.h"
-#include "keyvalue.h"
+#include "base.h"
+#include "burl.h"
+#include "http_kv.h"
 #include "log.h"
 #include "sock_addr.h"
 
@@ -649,7 +651,9 @@ int http_request_parse(server *srv, connection *con) {
 
 				/* check uri for invalid characters */
 				jlen = buffer_string_length(con->request.uri);
-				if (http_header_strict) {
+				if ((con->conf.http_parseopts & HTTP_PARSEOPT_URL_NORMALIZE_CTRLS_REJECT)) {
+					j = jlen; /* URI will be checked in http_response_prepare() */
+				} else if (http_header_strict) {
 					for (j = 0; j < jlen && request_uri_is_valid_char(con->request.uri->ptr[j]); j++) ;
 				} else {
 					char *z = memchr(con->request.uri->ptr, '\0', jlen);
@@ -1114,30 +1118,6 @@ int http_request_parse(server *srv, connection *con) {
 									ds->free((data_unset*) ds);
 									ds = NULL;
 								}
-							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("Range")))) {
-								if (!con->request.http_range) {
-									/* bytes=.*-.* */
-
-									if (0 == strncasecmp(ds->value->ptr, "bytes=", 6) &&
-									    NULL != strchr(ds->value->ptr+6, '-')) {
-
-										/* if dup, only the first one will survive */
-										con->request.http_range = ds->value->ptr + 6;
-									}
-								} else {
-									con->http_status = 400;
-									con->keep_alive = 0;
-
-									if (srv->srvconf.log_request_header_on_error) {
-										log_error_write(srv, __FILE__, __LINE__, "s",
-												"duplicate Range-header -> 400");
-										log_error_write(srv, __FILE__, __LINE__, "Sb",
-												"request-header:\n",
-												con->request.request);
-									}
-									array_insert_unique(con->request.headers, (data_unset *)ds);
-									return 0;
-								}
 							}
 
 							if (ds) array_insert_unique(con->request.headers, (data_unset *)ds);
